require('./utils');

var fs		= require('fs'),
path			= require('path'),
controller	= require('./controller');

var App = function() {

	//default paths
	this.paths			= {
		root: process.cwd(),
		config: './config',
		controllers: './controllers',
		models: './models',
		views: './views',
		public: './public',
		layouts: './views/_layouts'
	};
	
	//defaults engine
	this.engine		= {
		enconding: 'utf8',
		template: { object: require('ejs'), extension: 'ejs' },
		stylesheet: { extension: 'css' }
	};
	
	this.acceptExtensions = {
		img:		[ '.ico', '.jpg', '.png' ],
		script:	[ '.css', '.js' ]
	};
	
	//default controllers
	this.controllers	= {}
	
	//default routes
	this.routes		={};
};

App.prototype = {
	
	
	setup: function() {
		var context = this;
		
		return fs.readdir(this.paths.config, function(err, files){
			
			if (!err){
				var indexMapper	= files.indexOf('mapper.js'),
				indexCore				= files.indexOf('core.js');
				
				if (indexMapper > -1){
					
					var mapper	= require(path.normalize( context.paths.root + '/' + context.paths.config + '/' + files[indexMapper]));
					if ( typeof mapper == 'function' )
						mapper	= new mapper();
					if (mapper.paths)
						context.paths.extend(mapper.paths);
					
				}
				
				if(indexCore > -1) {
					var core			= require(path.normalize( context.paths.root + '/' + context.paths.config + '/' + files[indexCore]));
					this.engine		= core.engine;
					this.stylesheet	= core.stylesheet;
				
				}
			}
			context.setControllers();
		});
	},
	
	setControllers: function() {
		var context			= this;
		var controllerPath	= path.normalize( this.paths.root + '/' + this.paths.controllers);
		
		fs.readdir(controllerPath , function(err, files){
			if (err)
				throw err;
			files.forEach(function(file){
				
				fs.stat(controllerPath + '/' + file, function(err, stats) {
					if (stats.isFile()) {
						var _controller	= require(controllerPath + '/' + file);
						
						if ( typeof _controller == 'function' )
							_controller = new _controller();
												
						controller.actions.extend(_controller);
						for(a in controller.actions) {
							if ('function' == typeof controller.actions[a])
								controller.actions[a] = { action: controller.actions[a] }
							require('./action').call(controller.actions[a]);
						}
						context.controllers[file.split('.')[0]]	= controller;
					}
				});
			});
		});
	},
	
	dispatcher: function(request, response) {
		var context	= this;
		var url			= request.url.split('/'),
		parameters	= [];
		url.shift();
		
		while( url[url.length - 1] == '') url.pop();
		
		if( url.length > 0 ) {
			if (	url[url.length-1].match( this.acceptExtensions.script.join('$|') + '$' )
			||		url[url.length-1].match( this.acceptExtensions.img.join('$|') + '$' ) ) {
				
				fs.readFile( path.normalize( this.paths.root + '/' + this.paths.public + '/' + url.join('/') ), function(err, data) {
					if(!err) {
						response.writeHead(200);
						response.end( data );
					}
				} );
			}
		}
		else {
			while(url.length > 2) parameters.unshift(url.pop());
			
			if (!url[0])
				url.push('home');
			if (!url[1])
				url.push('index');
					
			if ( this.controllers[url[0]] && this.controllers[url[0]].actions[url[1]] ) {
				
				var _action	= this.controllers[url[0]].actions[url[1]];
				_action.action.apply(_action, parameters);
				
				//adding view path	
				url.unshift(this.paths.views);
				
				//Case change view
				if (_action.view)
					url[2] = _action.view;
				
				response.writeHead(200);
				response.end( this.render(url.join('/'), {
					data: _action.get(),
					layout: _action.layout || 'layout',
					title: _action.title || null,
					stylesheet: _action.stylesheet || null,
					script: _action.script || null
				}) );
			}
		}
		//else if (url) {
		//	this.render(this.paths.views + '/' + '400');
		//}
	},
	
	render: function(url, options) {
		
		var html,
		page	= fs.readFileSync( path.normalize( this.paths.root + '/' + url + '.' + this.engine.template.extension ), this.engine.enconding);
		data		= options.data;
		
		html	= this.engine.template.object.render( page, { locals: data });
		
		if (options && options.layout) {
			data		= {
				title : options.title,
				stylesheet : options.stylesheet,
				script: options.script,
				content: html
			};
			
			page	= fs.readFileSync( path.normalize( this.paths.root + '/' + this.paths.layouts + '/' + options.layout + '.' + this.engine.template.extension ), this.engine.enconding );
			html		= this.engine.template.object.render( page, { locals: data });
		}
		
		return html;
	}
}





















module.exports = exports = App;